(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = global || self, global.tsProve = factory());
}(this, (function () { 'use strict';

  /**
   * Curried wrappers for building type guards
   */
  var is = function (fn) { return function (value) { return fn(value); }; };
  var isEq = function (x) { return is(function (y) { return y === x; }); };
  /**
   * Generic type guards
   */
  var isString = is(function (value) { return typeof value === 'string'; });
  var isNumber = is(function (value) { return typeof value === 'number'; });
  var isFunction = is(function (value) { return typeof value === 'function'; });
  var isArray = is(Array.isArray);
  var isShape = is(function (value) { return typeof value === 'object' && !isFunction(value) && !isArray(value) && value !== null; });
  var isNull = isEq(null);
  var isUndefined = isEq(undefined);
  var hasKey = function (k, x) {
      return !!(x && !isUndefined(x[k]));
  };
  /** @internal
   * Returns a string value for any type
   */
  var toString = function (val) {
      return isString(val)
          ? val
          : isShape(val) || isArray(val)
              ? errorJ(val)
              : isNull(val)
                  ? 'null'
                  : isUndefined(val)
                      ? 'undefined'
                      : val.toString();
  };
  var errorT = function (expected, received) {
      return "Expected " + expected + " Recieved " + toString(received);
  };
  var errorJ = function (x) {
      return JSON.stringify(isArray(x) ? x.map(function (x, i) { return "[" + i + "] " + toString(x); }) : x, function (_key, value) { return (isString(value) ? value.replace(/\n/gim, "\n  ") : value); }, 2)
          .replace(/"/gim, '')
          .replace(/\\n/gim, "\n");
  };
  /**
   * TS Prove type guards
   */
  var isError = is(function (x) { return isString(x && x[0]); });

  /*! *****************************************************************************
  Copyright (c) Microsoft Corporation. All rights reserved.
  Licensed under the Apache License, Version 2.0 (the "License"); you may not use
  this file except in compliance with the License. You may obtain a copy of the
  License at http://www.apache.org/licenses/LICENSE-2.0

  THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
  WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
  MERCHANTABLITY OR NON-INFRINGEMENT.

  See the Apache Version 2.0 License for specific language governing permissions
  and limitations under the License.
  ***************************************************************************** */

  var __assign = function() {
      __assign = Object.assign || function __assign(t) {
          for (var s, i = 1, n = arguments.length; i < n; i++) {
              s = arguments[i];
              for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
          }
          return t;
      };
      return __assign.apply(this, arguments);
  };

  function __spreadArrays() {
      for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
      for (var r = Array(s), k = 0, i = 0; i < il; i++)
          for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
              r[k] = a[j];
      return r;
  }

  /** @internal
   * Validator result wrapper
   */
  var result = function (x, err, res) {
      return x ? [null, res] : [err, res];
  };
  /**
   * TS Prove validators
   */
  var _string = function () { return function (x) { return result(isString(x), errorT('string', x), x); }; };
  var _number = function () { return function (x) { return result(isNumber(x), errorT('number', x), x); }; };
  var _null = function () { return function (x) { return result(isNull(x), errorT('null', x), x); }; };
  var _undefined = function () { return function (x) {
      return result(isUndefined(x), errorT('undefined', x), x);
  }; };
  var _any = function () { return function (x) { return result(true, '', x); }; };
  var _unknown = function () { return function (x) {
      return result(true, '', x);
  }; };
  /**
   * Determin that type is on of an array of types and return an intersection type
   * @param P Prove<T>[]
   * @return (v: unknown) => Success<T[number]> | Failure
   */
  var _or = function (x) { return function (y) {
      return result(x.reduce(function (a, b) { return a === true || !b(y)[0]; }, false) === true, "Nothing matched " + toString(y), y);
  }; };
  /**
   * Determin that every element in an array is of type
   * @param P Prove<T>
   * @return (v: unknown) => Success<T[]> | Failure
   */
  var _array = function (x) { return function (y) {
      if (!isArray(y))
          return [errorT('array', y), y];
      var result = y.reduce(function (a, b) { return (isError(x(b)) ? __spreadArrays(a, [x(b)[0]]) : a); }, []);
      if (result.length > 0)
          return [errorJ(result), y];
      return [null, y];
  }; };
  /**
   * Execute a key value object of validators
   * @param P { [x: string]: Prove<T> }
   * @return (v: unknown) => Success<{ [x: string]: T }> | Failure
   */
  var _shape = function (shpe) { return function (y) {
      var result = Object.keys(shpe).reduce(function (all, key) {
          var _a, _b;
          if (!hasKey(key, y))
              return [true, __assign(__assign({}, all[1]), (_a = {}, _a[key] = '__missing__', _a))];
          var res = shpe[key](y[key]);
          if (isError(res))
              return [true, __assign(__assign({}, all[1]), (_b = {}, _b[key] = res[0], _b))];
          return all;
      }, [false, {}]);
      if (result[0])
          return [errorJ(result[1]), y];
      return [null, y];
  }; };

  var tsProve = {
      string: _string,
      number: _number,
      null: _null,
      undefined: _undefined,
      any: _any,
      unknown: _unknown,
      or: _or,
      array: _array,
      shape: _shape,
      s: _string,
      n: _number,
      o: _shape,
      a: _array,
  };

  return tsProve;

})));
//# sourceMappingURL=ts-prove.umd.js.map
